@@ -928,7 +928,7 @@ fn compile_with_exec(
         let pkg = &unit.pkg;
         let capi_config = load_manifest_capi_config(pkg, rustc_target)?;
         let name = &capi_config.library.name;
-        let install_paths = InstallPaths::new(name, args, &capi_config);
+        let install_paths = InstallPaths::new(name, rustc_target, args, &capi_config);
         let pkg_rustflags = &capi_config.library.rustflags;
 
         let mut leaf_args: Vec<String> = rustc_target
@@ -1018,9 +1018,15 @@ impl CPackage {
 
         let name = &capi_config.library.name;
 
-        let install_paths = InstallPaths::new(name, args, &capi_config);
-        let build_targets =
-            BuildTargets::new(name, rustc_target, root_output, libkinds, &capi_config)?;
+        let install_paths = InstallPaths::new(name, rustc_target, args, &capi_config);
+        let build_targets = BuildTargets::new(
+            name,
+            rustc_target,
+            root_output,
+            libkinds,
+            &capi_config,
+            args.get_flag("meson"),
+        )?;
 
         let finger_print = FingerPrint::new(&id, root_output, &build_targets, &install_paths);
 
@@ -1043,15 +1049,15 @@ pub fn cbuild(
 ) -> anyhow::Result<(Vec<CPackage>, CompileOptions)> {
     let rustc = config.load_global_rustc(Some(ws))?;
     let targets = args.targets()?;
-    let target = match targets.len() {
-        0 => rustc.host.to_string(),
-        1 => targets[0].to_string(),
+    let (target, is_target_overridden) = match targets.len() {
+        0 => (rustc.host.to_string(), false),
+        1 => (targets[0].to_string(), true),
         _ => {
             anyhow::bail!("Multiple targets not supported yet");
         }
     };
 
-    let rustc_target = target::Target::new(&target)?;
+    let rustc_target = target::Target::new(Some(&target), is_target_overridden)?;
 
     let default_kind = || match (rustc_target.os.as_str(), rustc_target.env.as_str()) {
         ("none", _) | (_, "musl") => vec!["staticlib"],
@@ -1209,6 +1215,7 @@ pub fn cbuild(
                     &root_output,
                     &libkinds,
                     capi_config,
+                    args.get_flag("meson"),
                 )?;
 
                 if let (Some(from_static_lib), Some(to_static_lib)) = (
@@ -1223,6 +1230,12 @@ pub fn cbuild(
                 ) {
                     copy(from_shared_lib, to_shared_lib)?;
                 }
+                if let (Some(from_debug_info), Some(to_debug_info)) = (
+                    from_build_targets.debug_info.as_ref(),
+                    build_targets.debug_info.as_ref(),
+                ) {
+                    copy(from_debug_info, to_debug_info)?;
+                }
             }
 
             cpkg.finger_print.static_libs = static_libs;
